function AddExtraFrameGRF
% This function adds an extra time frame to the end of a marker motion file
% because opensim will often ignore the final time frame during inverse
% kinematics

originalDir = cd;
cd('Data');

files = dir('*GRF.mot');
nfiles = length(files);

for i = 1:nfiles

infile = files(i).name
    
fid = fopen(infile);
header1 = fgetl(fid);
header2 = fgetl(fid);
header3 = fgetl(fid);
header4 = fgetl(fid);
header5 = fgetl(fid);
header6 = fgetl(fid);
header7 = fgetl(fid);

% Read marker data as one long column
data = fscanf(fid,'%f');
fclose(fid);

% Reshape marker data based on number of rows and
% columns of marker data specified in the input data
% file, with two extra columns for frame and time
nrows = str2num(strrep(header3,'nRows=',''));
ncols = str2num(strrep(header4,'nColumns=',''));
data = reshape(data,ncols,nrows)';

% Separate frame and time vectors from marker data
time = data(:,1);
grf_data = data(:,2:ncols);

step = mean(diff(time));
time = [time; time(end,:)+step];
grf_data = [grf_data; grf_data(end,:)];

outputGRFmot(infile,grf_data,time)

end

cd(originalDir);
end

function outputGRFmot(outFile,GRFData,Time)

Data = [Time GRFData];

Header = [outFile sprintf('\n') ...
    'version=1' sprintf('\n') ...
    'nRows=' num2str(size(Data,1)) sprintf('\n') ...
    'nColumns=' num2str(size(Data,2)) sprintf('\n') ...
    'inDegrees=no' sprintf('\n') ...
    'endheader' sprintf('\n') ...
    'time' sprintf('\t') 'F1x' sprintf('\t') 'F1y' sprintf('\t') 'F1z' sprintf('\t') 'M1x' sprintf('\t')...
    'M1y' sprintf('\t') 'M1z' sprintf('\t') 'F2x' sprintf('\t') 'F2y' sprintf('\t') 'F2z' sprintf('\t')...
    'M2x' sprintf('\t') 'M2y' sprintf('\t') 'M2z' sprintf('\t') 'EC1x' sprintf('\t')...
    'EC1y' sprintf('\t') 'EC1z' sprintf('\t') 'EC2x' sprintf('\t')...
    'EC2y' sprintf('\t') 'EC2z' sprintf('\n')];

nRows = size(Data,1);
nColumns = size(Data,2);

DataString = '';
for j = 1:nRows
    for k = 1:nColumns
        if k ~= nColumns
            DataString = [DataString num2str(Data(j,k)) sprintf('\t')];
        else
            DataString = [DataString num2str(Data(j,k)) sprintf('\n')];
        end
    end
end

fid = fopen(outFile,'w');
fprintf(fid,'%s',[Header DataString]);
fclose(fid);
end