function matchStimToNonStimTrials
% Find similar stim and non-stim trials, and organize trials into
% structures
origDir = cd;
cd('Data');

% Load all non-stim trial ID Data into a structure
nonStimIDFiles = dir('Trial*_id.sto');
numNonStimTrials = length(nonStimIDFiles);

for iii = 1:numNonStimTrials
    [nonStimTrials(iii).IDData,~,nonStimTrials(iii).time] = readMotFile(nonStimIDFiles(iii).name);
end

% Load all stim trial ID Data into a structure
stimIDFiles = dir('FES*_id.sto');
numStimTrials = length(stimIDFiles);

for iii = 1:numStimTrials
    [stimTrials(iii).IDData,~,stimTrials(iii).time] = readMotFile(stimIDFiles(iii).name);
end

% Find RMS error between each set of a non-stim and stimm trial
RMSErrors = zeros(numStimTrials,numNonStimTrials,12);
totalRMSErrors = zeros(numStimTrials,numNonStimTrials);
for iii = 1:numStimTrials
    for jjj = 1:numNonStimTrials
        % [7:10,12:18,20:29,33:36] [1:6,8:13] = [7:10,12,13,15:18,20,21]
        % NOTE (Nathan): Indexing is because we only really care about leg ID torques
        RMSErrors(iii,jjj,:) = sqrt(mean((stimTrials(iii).IDData(1:141,[7:10,12,13,15:18,20,21]) - nonStimTrials(jjj).IDData(1:141,[7:10,12,13,15:18,20,21])).^2,1));
        totalRMSErrors(iii,jjj) = sum(squeeze(RMSErrors(iii,jjj,:)));
    end
end

% Match indices to lowest RMS error for each stim trial
bestMatch = zeros(1,numStimTrials);
totalRMSMatch = zeros(1,numStimTrials);
for iii = 1:numStimTrials
    [totalRMSMatch(iii),bestMatch(iii)] = min(totalRMSErrors(iii,:));
end

stimTrialNames = cell(numStimTrials,1);
nonStimTrialNames = cell(numNonStimTrials,1);
for iii = 1:numStimTrials
    stimTrialNames{iii} = stimIDFiles(iii).name;
end
for iii = 1:numNonStimTrials
    nonStimTrialNames{iii} = nonStimIDFiles(iii).name;
end

cd(origDir);
save stimToNonStimTrialMatching.mat totalRMSMatch bestMatch stimTrialNames nonStimTrialNames

end



%% Function Definitions
function [Data, ColumnLabels, Time] = readMotFile(inFile)

%open file
fid = fopen(inFile);

%skip initial lines
line = 'asdf';
while ~strcmpi(line,'ENDHEADER')
    line = fgetl(fid);
    if length(line)>8
        line = line(1:9);
    end
end

% Find the labels on the columns
ColumnLabels = fgetl(fid);
ColumnLabels = textscan(ColumnLabels, '%t');
ColumnLabels = ColumnLabels{1}';

% Read numeric data
Data = fscanf(fid, '%f');

% Find length of data
nColumns = length(ColumnLabels);
nRows = length(Data)/nColumns;

% Reshape data into matrix
Data = reshape(Data, nColumns, nRows)';

% Extract time vector
Time = Data(:,1);
Data(:,1) = [];

% Remove time label from column labels
ColumnLabels(1) = [];

fclose(fid);
end
